@@ -22,7 +22,7 @@ import typing
 from decimal import Decimal
 from solana.publickey import PublicKey
 
-from .client import BetterClient
+from .client import BetterClient, ClusterUrlData, TransactionStatusCollector, NullTransactionStatusCollector
 from .constants import MangoConstants, DATA_PATH
 from .context import Context
 from .idsjsonmarketlookup import IdsJsonMarketLookup
@@ -53,6 +53,25 @@ from .serummarketlookup import SerumMarketLookup
 # A `ContextBuilder` class to allow building `Context` objects without introducing circular dependencies.
 #
 class ContextBuilder:
+
+    # Utility class for parsing cluster-url command line parameter
+    #
+    class ParseClusterUrls(argparse.Action):
+        cluster_urls: typing.List[ClusterUrlData] = []
+
+        def __call__(self, parser: argparse.ArgumentParser, namespace: object, values: typing.Any, option_string: typing.Optional[str] = None) -> None:
+            if values:
+                if len(values) == 1:
+                    self.cluster_urls.append(ClusterUrlData(rpc=values[0]))
+                elif len(values) == 2:
+                    self.cluster_urls.append(ClusterUrlData(rpc=values[0], ws=values[1]))
+                else:
+                    raise parser.error(
+                        'Argument --cluster-url permits maximal two parameters. The first one configures HTTP connection url, the second one '
+                        'configures the WS connection url. Example: --cluster-url https://localhost:8181 wss://localhost:8282'
+                    )
+                setattr(namespace, self.dest, self.cluster_urls)
+
     # Configuring a `Context` is a common operation for command-line programs and can involve a
     # lot of duplicate code.
     #
@@ -63,8 +82,8 @@ class ContextBuilder:
         parser.add_argument("--name", type=str, default="Mango Explorer",
                             help="Name of the program (used in reports and alerts)")
         parser.add_argument("--cluster-name", type=str, default=None, help="Solana RPC cluster name")
-        parser.add_argument("--cluster-url", type=str, action="append", default=[],
-                            help="Solana RPC cluster URL (can be specified multiple times to provide failover when one errors)")
+        parser.add_argument("--cluster-url", nargs='*', type=str, action=ContextBuilder.ParseClusterUrls, default=[],
+                            help="Solana RPC cluster URL (can be specified multiple times to provide failover when one errors; optional second parameter value defines websocket connection)")
         parser.add_argument("--group-name", type=str, default=None, help="Mango group name")
         parser.add_argument("--group-address", type=PublicKey, default=None, help="Mango group address")
         parser.add_argument("--mango-program-address", type=PublicKey, default=None, help="Mango program address")
@@ -76,6 +95,8 @@ class ContextBuilder:
                             help="Encoding to request when receiving data from Solana (options are 'base58' (slow), 'base64', 'base64+zstd', or 'jsonParsed')")
         parser.add_argument("--blockhash-cache-duration", type=int,
                             help="How long (in seconds) to cache 'recent' blockhashes")
+        parser.add_argument("--http-request-timeout", type=float, default=20,
+                            help="What is the timeout for HTTP requests to when calling to RPC nodes (in seconds), -1 means no timeout")
         parser.add_argument("--stale-data-pause-before-retry", type=Decimal,
                             help="How long (in seconds, e.g. 0.1) to pause after retrieving stale data before retrying")
         parser.add_argument("--stale-data-maximum-retries", type=int,
@@ -98,7 +119,7 @@ class ContextBuilder:
     def from_command_line_parameters(args: argparse.Namespace) -> Context:
         name: typing.Optional[str] = args.name
         cluster_name: typing.Optional[str] = args.cluster_name
-        cluster_urls: typing.Optional[typing.Sequence[str]] = args.cluster_url
+        cluster_urls: typing.Optional[typing.Sequence[ClusterUrlData]] = args.cluster_url
         group_name: typing.Optional[str] = args.group_name
         group_address: typing.Optional[PublicKey] = args.group_address
         mango_program_address: typing.Optional[PublicKey] = args.mango_program_address
@@ -107,6 +128,7 @@ class ContextBuilder:
         commitment: typing.Optional[str] = args.commitment
         encoding: typing.Optional[str] = args.encoding
         blockhash_cache_duration: typing.Optional[int] = args.blockhash_cache_duration
+        http_request_timeout: typing.Optional[float] = args.http_request_timeout
         stale_data_pause_before_retry: typing.Optional[Decimal] = args.stale_data_pause_before_retry
         stale_data_maximum_retries: typing.Optional[int] = args.stale_data_maximum_retries
         gma_chunk_size: typing.Optional[Decimal] = args.gma_chunk_size
@@ -125,7 +147,7 @@ class ContextBuilder:
             actual_stale_data_pauses_before_retry = [float(pause)] * retries
 
         context: Context = ContextBuilder.build(name, cluster_name, cluster_urls, skip_preflight, commitment,
-                                                encoding, blockhash_cache_duration,
+                                                encoding, blockhash_cache_duration, http_request_timeout,
                                                 actual_stale_data_pauses_before_retry,
                                                 group_name, group_address, mango_program_address,
                                                 serum_program_address, gma_chunk_size, gma_chunk_pause,
@@ -142,7 +164,7 @@ class ContextBuilder:
     def from_group_name(context: Context, group_name: str) -> Context:
         return ContextBuilder.build(context.name, context.client.cluster_name, context.client.cluster_urls,
                                     context.client.skip_preflight, context.client.commitment,
-                                    context.client.encoding, context.client.blockhash_cache_duration,
+                                    context.client.encoding, context.client.blockhash_cache_duration, None,
                                     context.client.stale_data_pauses_before_retry,
                                     group_name, None, None, None,
                                     context.gma_chunk_size, context.gma_chunk_pause,
@@ -151,7 +173,7 @@ class ContextBuilder:
     @staticmethod
     def forced_to_devnet(context: Context) -> Context:
         cluster_name: str = "devnet"
-        cluster_url: str = MangoConstants["cluster_urls"][cluster_name]
+        cluster_url: ClusterUrlData = ClusterUrlData(rpc=MangoConstants["cluster_urls"][cluster_name])
         fresh_context = copy.copy(context)
         fresh_context.client = BetterClient.from_configuration(context.name,
                                                                cluster_name,
@@ -160,15 +182,17 @@ class ContextBuilder:
                                                                context.client.skip_preflight,
                                                                context.client.encoding,
                                                                context.client.blockhash_cache_duration,
+                                                               -1,
                                                                context.client.stale_data_pauses_before_retry,
-                                                               context.client.instruction_reporter)
+                                                               context.client.instruction_reporter,
+                                                               context.client.transaction_status_collector)
 
         return fresh_context
 
     @staticmethod
     def forced_to_mainnet_beta(context: Context) -> Context:
         cluster_name: str = "mainnet"
-        cluster_url: str = MangoConstants["cluster_urls"][cluster_name]
+        cluster_url: ClusterUrlData = ClusterUrlData(rpc=MangoConstants["cluster_urls"][cluster_name])
         fresh_context = copy.copy(context)
         fresh_context.client = BetterClient.from_configuration(context.name,
                                                                cluster_name,
@@ -177,21 +201,26 @@ class ContextBuilder:
                                                                context.client.skip_preflight,
                                                                context.client.encoding,
                                                                context.client.blockhash_cache_duration,
+                                                               -1,
                                                                context.client.stale_data_pauses_before_retry,
-                                                               context.client.instruction_reporter)
+                                                               context.client.instruction_reporter,
+                                                               context.client.transaction_status_collector)
 
         return fresh_context
 
     @staticmethod
     def build(name: typing.Optional[str] = None, cluster_name: typing.Optional[str] = None,
-              cluster_urls: typing.Optional[typing.Sequence[str]] = None, skip_preflight: bool = False,
+              cluster_urls: typing.Optional[typing.Sequence[ClusterUrlData]] = None,
+              skip_preflight: bool = False,
               commitment: typing.Optional[str] = None, encoding: typing.Optional[str] = None,
               blockhash_cache_duration: typing.Optional[int] = None,
+              http_request_timeout: typing.Optional[float] = None,
               stale_data_pauses_before_retry: typing.Optional[typing.Sequence[float]] = None,
               group_name: typing.Optional[str] = None, group_address: typing.Optional[PublicKey] = None,
               program_address: typing.Optional[PublicKey] = None, serum_program_address: typing.Optional[PublicKey] = None,
               gma_chunk_size: typing.Optional[Decimal] = None, gma_chunk_pause: typing.Optional[Decimal] = None,
-              token_filename: str = SPLTokenLookup.DefaultDataFilepath) -> "Context":
+              token_filename: str = SPLTokenLookup.DefaultDataFilepath,
+              transaction_status_collector: TransactionStatusCollector = NullTransactionStatusCollector()) -> "Context":
         def __public_key_or_none(address: typing.Optional[str]) -> typing.Optional[PublicKey]:
             if address is not None and address != "":
                 return PublicKey(address)
@@ -212,14 +241,15 @@ class ContextBuilder:
         actual_encoding: str = encoding or "base64"
         actual_blockhash_cache_duration: int = blockhash_cache_duration or 0
         actual_stale_data_pauses_before_retry: typing.Sequence[float] = stale_data_pauses_before_retry or []
+        actual_http_request_timeout: float = http_request_timeout or -1
 
-        actual_cluster_urls: typing.Optional[typing.Sequence[str]] = cluster_urls
+        actual_cluster_urls: typing.Optional[typing.Sequence[ClusterUrlData]] = cluster_urls
         if actual_cluster_urls is None or len(actual_cluster_urls) == 0:
             cluster_url_from_environment: typing.Optional[str] = os.environ.get("CLUSTER_URL")
             if cluster_url_from_environment is not None and cluster_url_from_environment != "":
-                actual_cluster_urls = [cluster_url_from_environment]
+                actual_cluster_urls = [ClusterUrlData(rpc=cluster_url_from_environment)]
             else:
-                actual_cluster_urls = [MangoConstants["cluster_urls"][actual_cluster]]
+                actual_cluster_urls = [ClusterUrlData(rpc=MangoConstants["cluster_urls"][actual_cluster])]
 
         actual_skip_preflight: bool = skip_preflight
         actual_group_name: str = group_name or os.environ.get("GROUP_NAME") or default_group_data["name"]
@@ -307,4 +337,4 @@ class ContextBuilder:
                 devnet_serum_market_lookup])
         market_lookup: MarketLookup = all_market_lookup
 
-        return Context(actual_name, actual_cluster, actual_cluster_urls, actual_skip_preflight, actual_commitment, actual_encoding, actual_blockhash_cache_duration, actual_stale_data_pauses_before_retry, actual_program_address, actual_serum_program_address, actual_group_name, actual_group_address, actual_gma_chunk_size, actual_gma_chunk_pause, instrument_lookup, market_lookup)
+        return Context(actual_name, actual_cluster, actual_cluster_urls, actual_skip_preflight, actual_commitment, actual_encoding, actual_blockhash_cache_duration, actual_http_request_timeout, actual_stale_data_pauses_before_retry, actual_program_address, actual_serum_program_address, actual_group_name, actual_group_address, actual_gma_chunk_size, actual_gma_chunk_pause, instrument_lookup, market_lookup, transaction_status_collector)
