@@ -42,15 +42,15 @@ from .wallet import Wallet
 # on initial setup in the `load()` method.
 #
 class SpotMarketInstructionBuilder(MarketInstructionBuilder):
-    def __init__(self, context: Context, wallet: Wallet, group: Group, account: Account,
-                 spot_market: SpotMarket, raw_market: PySerumMarket, market_index: int,
-                 fee_discount_token_address: PublicKey) -> None:
+    def __init__(self, context: Context, wallet: Wallet, spot_market: SpotMarket,
+                 group: Group, account: Account, raw_market: PySerumMarket,
+                 market_index: int, fee_discount_token_address: PublicKey) -> None:
         super().__init__()
         self.context: Context = context
         self.wallet: Wallet = wallet
+        self.spot_market: SpotMarket = spot_market
         self.group: Group = group
         self.account: Account = account
-        self.spot_market: SpotMarket = spot_market
         self.raw_market: PySerumMarket = raw_market
         self.market_index: int = market_index
         self.fee_discount_token_address: PublicKey = fee_discount_token_address
@@ -58,7 +58,7 @@ class SpotMarketInstructionBuilder(MarketInstructionBuilder):
         self.open_orders_address: typing.Optional[PublicKey] = self.account.spot_open_orders_by_index[self.market_index]
 
     @staticmethod
-    def load(context: Context, wallet: Wallet, group: Group, account: Account, spot_market: SpotMarket) -> "SpotMarketInstructionBuilder":
+    def load(context: Context, wallet: Wallet, spot_market: SpotMarket, group: Group, account: Account) -> "SpotMarketInstructionBuilder":
         raw_market: PySerumMarket = PySerumMarket.load(
             context.client.compatible_client, spot_market.address, context.serum_program_address)
 
@@ -76,7 +76,7 @@ class SpotMarketInstructionBuilder(MarketInstructionBuilder):
         slot = group.slot_by_spot_market_address(spot_market.address)
         market_index = slot.index
 
-        return SpotMarketInstructionBuilder(context, wallet, group, account, spot_market, raw_market, market_index, fee_discount_token_address)
+        return SpotMarketInstructionBuilder(context, wallet, spot_market, group, account, raw_market, market_index, fee_discount_token_address)
 
     def build_cancel_order_instructions(self, order: Order, ok_if_missing: bool = False) -> CombinableInstructions:
         if self.open_orders_address is None:
@@ -118,9 +118,14 @@ class SpotMarketInstructionBuilder(MarketInstructionBuilder):
             if oo not in distinct_addresses:
                 distinct_addresses += [oo]
 
+        if len(distinct_addresses) > limit:
+            self._logger.warn(
+                f"Cranking limited to {limit} of {len(distinct_addresses)} addresses waiting to be cranked.")
+
         limited_addresses = distinct_addresses[0:min(int(limit), len(distinct_addresses))]
         limited_addresses.sort(key=encode_public_key_for_sorting)
 
+        self._logger.debug(f"About to crank {len(limited_addresses)} addresses: {limited_addresses}")
         return build_serum_consume_events_instructions(self.context, self.spot_market.address, self.raw_market.state.event_queue(), limited_addresses, int(limit))
 
     def build_redeem_instructions(self) -> CombinableInstructions:
@@ -138,19 +143,25 @@ class SpotMarketInstructionBuilder(MarketInstructionBuilder):
 # This class puts trades on the Serum orderbook. It doesn't do anything complicated.
 #
 class SpotMarketOperations(MarketOperations):
-    def __init__(self, context: Context, wallet: Wallet, group: Group, account: Account,
-                 spot_market: SpotMarket, market_instruction_builder: SpotMarketInstructionBuilder) -> None:
-        super().__init__(spot_market)
+    def __init__(self, context: Context, wallet: Wallet, account: Account,
+                 market_instruction_builder: SpotMarketInstructionBuilder) -> None:
+        super().__init__(market_instruction_builder.spot_market)
         self.context: Context = context
         self.wallet: Wallet = wallet
-        self.group: Group = group
         self.account: Account = account
-        self.spot_market: SpotMarket = spot_market
         self.market_instruction_builder: SpotMarketInstructionBuilder = market_instruction_builder
 
-        self.market_index: int = group.slot_by_spot_market_address(spot_market.address).index
+        self.market_index: int = self.group.slot_by_spot_market_address(self.spot_market.address).index
         self.open_orders_address: typing.Optional[PublicKey] = self.account.spot_open_orders_by_index[self.market_index]
 
+    @property
+    def spot_market(self) -> SpotMarket:
+        return self.market_instruction_builder.spot_market
+
+    @property
+    def group(self) -> Group:
+        return self.market_instruction_builder.group
+
     def cancel_order(self, order: Order, ok_if_missing: bool = False) -> typing.Sequence[str]:
         self._logger.info(f"Cancelling {self.spot_market.symbol} order {order}.")
         signers: CombinableInstructions = CombinableInstructions.from_wallet(self.wallet)
@@ -161,15 +172,17 @@ class SpotMarketOperations(MarketOperations):
 
         return (signers + cancel + crank + settle).execute(self.context)
 
-    def place_order(self, order: Order) -> Order:
+    def place_order(self, order: Order, crank_limit: Decimal = Decimal(5)) -> Order:
         client_id: int = self.context.generate_client_id()
         signers: CombinableInstructions = CombinableInstructions.from_wallet(self.wallet)
+        if order.reduce_only:
+            self._logger.warning("Ignoring reduce_only flag on order because spot markets don't support it.")
         order_with_client_id: Order = order.with_client_id(client_id).with_owner(
             self.open_orders_address or SYSTEM_PROGRAM_ADDRESS)
         self._logger.info(f"Placing {self.spot_market.symbol} order {order}.")
         place: CombinableInstructions = self.market_instruction_builder.build_place_order_instructions(
             order_with_client_id)
-        crank: CombinableInstructions = self._build_crank(add_self=True)
+        crank: CombinableInstructions = self._build_crank(limit=crank_limit, add_self=True)
         settle: CombinableInstructions = self.market_instruction_builder.build_settle_instructions()
 
         transaction_ids = (signers + place + crank + settle).execute(self.context)
@@ -227,6 +240,8 @@ class SpotMarketOperations(MarketOperations):
         if len(open_orders_to_crank) == 0:
             return CombinableInstructions.empty()
 
+        self._logger.debug(
+            f"Building crank instruction with {len(open_orders_to_crank)} public keys, throttled to {limit}")
         return self.market_instruction_builder.build_crank_instructions(open_orders_to_crank, limit)
 
     def __str__(self) -> str:
